package interfaz;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.SwingConstants;
import javax.swing.text.NumberFormatter;

import bloques.Bloque;
import bloques.BloqueBasico;

/**
 * <p>Title: VariarParametros.java</p>
 * <p>Description: Las instancias de sta clase son dilogos que permiten al
 * usuario introducir los datos para la obtencin de resultados con variacin
 * de parmetros
 * @author Diego Daz Garca
 * @version 1.0
 */

public class VariarParametros
    extends JDialog
    implements ActionListener {
  JLabel jLabel1 = new JLabel();
  JComboBox comboElementos = new JComboBox();
  JComboBox comboParametros = new JComboBox();
  JLabel jLabel2 = new JLabel();
  JLabel jLabel3 = new JLabel();
  JFormattedTextField campoValorInicial = new JFormattedTextField();
  JFormattedTextField campoValorFinal = new JFormattedTextField();
  JLabel jLabel4 = new JLabel();
  JLabel jLabel5 = new JLabel();
  JFormattedTextField campoIncremento = new JFormattedTextField();
  JButton botonCancelar = new JButton();
  JButton botonAceptar = new JButton();
  String elemento;
  Bloque bloquePadre;
  // Formato para tasa de fallos, MTTF, MTBF, etc.
  private NumberFormat formatoParametros;
  Opciones opcionesResolucion;
  FramePrincipal ventanaPrincipal;

  // Valores estticos para que se conserven entre invocaciones
  static private double valorInicial = 0;
  static private double valorFinal = 0;
  static private double incremento = 0;
  static private String elementoAnterior;
  static private int indiceParametroAnterior;

  public VariarParametros(FramePrincipal ventanaPrincipal, Bloque bloquePadre, Opciones
                          opcionesResolucion) {
    super(ventanaPrincipal, "Escoger variacin de parmetro", false);
    this.bloquePadre = bloquePadre;
    this.opcionesResolucion = opcionesResolucion;
    this.ventanaPrincipal = ventanaPrincipal;

    try {
      jbInit();
      pack();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
    campoValorInicial.setValue(new Double(valorInicial));
    campoValorFinal.setValue(new Double(valorFinal));
    campoIncremento.setValue(new Double(incremento));

    // Comprobar que si se ha guardado un bloque anterior como elemento
    // a variar, sigue existiendo. En caso contrario, poner "Tiempo"
    if (elementoAnterior != "Tiempo") {
      for (int i = 0; i < comboElementos.getItemCount(); i++) {
      	String nombre = (String) comboElementos.getItemAt(i);
      	if (nombre == elementoAnterior) {
      	  comboElementos.setSelectedIndex(i);
      	  // La siguiente comparacin es necesaria porque al pasar de reparable
      	  // a no reparable aparecen menos opciones en comboParametros
      	  if (indiceParametroAnterior < comboParametros.getItemCount())
      	    comboParametros.setSelectedIndex(indiceParametroAnterior);
      	}
      }
    }
  }

  private void jbInit() throws Exception {
    // se prepara el formato que van a tener los campos excepto para el tiempo
    formatoParametros = new DecimalFormat("#0.0############");
    NumberFormatter mascaraDecimal = new NumberFormatter(formatoParametros);
    mascaraDecimal.setOverwriteMode(false);
    mascaraDecimal.setAllowsInvalid(false);

    campoValorInicial = new JFormattedTextField(mascaraDecimal);
    campoValorFinal   = new JFormattedTextField(mascaraDecimal);
    campoIncremento   = new JFormattedTextField(mascaraDecimal);

    this.getContentPane().setLayout(null);
    jLabel1.setFont(new java.awt.Font("Dialog", 0, 12));
    jLabel1.setText("Elementos a Variar");
    jLabel1.setBounds(new Rectangle(23, 23, 117, 21));
    comboElementos.setFont(new java.awt.Font("Dialog", 0, 12));
    comboElementos.setBounds(new Rectangle(157, 23, 148, 21));
    comboParametros.setFont(new java.awt.Font("Dialog", 0, 12));
    comboParametros.setBounds(new Rectangle(157, 58, 148, 21));
    jLabel2.setBounds(new Rectangle(23, 58, 117, 21));
    jLabel2.setFont(new java.awt.Font("Dialog", 0, 12));
    jLabel2.setOpaque(false);
    jLabel2.setText("Parmetro");
    jLabel3.setFont(new java.awt.Font("Dialog", 0, 12));
    jLabel3.setText("Valor Inicial");
    jLabel3.setBounds(new Rectangle(76, 102, 123, 26));
    campoValorInicial.setBackground(Color.white);
    campoValorInicial.setFont(new java.awt.Font("Dialog", 0, 12));
    campoValorInicial.setForeground(Color.black);
    campoValorInicial.setDebugGraphicsOptions(0);
    campoValorInicial.setDoubleBuffered(false);
    campoValorInicial.setInputVerifier(null);
    campoValorInicial.setCaretColor(Color.black);
    campoValorInicial.setEditable(true);
    campoValorInicial.setColumns(0);
    campoValorInicial.setHorizontalAlignment(SwingConstants.RIGHT);
    campoValorInicial.setBounds(new Rectangle(205, 107, 98, 21));
    campoValorFinal.setBounds(new Rectangle(205, 136, 98, 21));
    campoValorFinal.setHorizontalAlignment(SwingConstants.RIGHT);
    campoValorFinal.setColumns(0);
    campoValorFinal.setEditable(true);
    campoValorFinal.setCaretColor(Color.black);
    campoValorFinal.setInputVerifier(null);
    campoValorFinal.setDoubleBuffered(false);
    campoValorFinal.setDebugGraphicsOptions(0);
    campoValorFinal.setForeground(Color.black);
    campoValorFinal.setFont(new java.awt.Font("Dialog", 0, 12));
    campoValorFinal.setBackground(Color.white);
    jLabel4.setBounds(new Rectangle(76, 131, 123, 26));
    jLabel4.setText("Valor Final");
    jLabel4.setFont(new java.awt.Font("Dialog", 0, 12));
    jLabel5.setFont(new java.awt.Font("Dialog", 0, 12));
    jLabel5.setText("Incremento");
    jLabel5.setBounds(new Rectangle(76, 159, 123, 26));
    campoIncremento.setBackground(Color.white);
    campoIncremento.setFont(new java.awt.Font("Dialog", 0, 12));
    campoIncremento.setForeground(Color.black);
    campoIncremento.setDebugGraphicsOptions(0);
    campoIncremento.setDoubleBuffered(false);
    campoIncremento.setInputVerifier(null);
    campoIncremento.setCaretColor(Color.black);
    campoIncremento.setEditable(true);
    campoIncremento.setColumns(0);
    campoIncremento.setHorizontalAlignment(SwingConstants.RIGHT);
    campoIncremento.setBounds(new Rectangle(205, 164, 98, 21));
    botonCancelar.setBounds(new Rectangle(205, 202, 100, 31));
    botonCancelar.setText("Cancelar");
    botonAceptar.setText("Aceptar");
    botonAceptar.setBounds(new Rectangle(95, 202, 100, 31));
    this.setResizable(false);
    this.getContentPane().add(jLabel1, null);
    this.getContentPane().add(comboElementos, null);
    this.getContentPane().add(jLabel2, null);
    this.getContentPane().add(comboParametros, null);
    this.getContentPane().add(jLabel3, null);
    this.getContentPane().add(campoValorInicial, null);
    this.getContentPane().add(jLabel4, null);
    this.getContentPane().add(campoValorFinal, null);
    this.getContentPane().add(jLabel5, null);
    this.getContentPane().add(campoIncremento, null);
    this.getContentPane().add(botonAceptar, null);
    this.getContentPane().add(botonCancelar, null);

    comboParametros.setEnabled(false);
    rellenarListaElementos();

    comboElementos.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        elemento = (String) ( (JComboBox) e.getSource()).getSelectedItem();
        actualizarComboParametros();
      }
    });

    botonAceptar.addActionListener(this);
    botonCancelar.addActionListener(this);
  }

  private void resolver(Bloque bloqueaResolver, String Elemento,
                        BloqueBasico bloque, String parametro) {

  	// Guardar los valores para restaurarlos cuando se vuelva a abrir la ventana
  	elementoAnterior = elemento;
  	indiceParametroAnterior = comboParametros.getSelectedIndex();

    valorInicial = ( (Number) campoValorInicial.getValue()).doubleValue();
    valorFinal = ( (Number) campoValorFinal.getValue()).doubleValue();
    incremento = ( (Number) campoIncremento.getValue()).doubleValue();
    crearResultados(bloqueaResolver, parametro, elemento, bloque, valorInicial,
                    valorFinal, incremento);
  }

  private void crearResultados(Bloque bloqueaResolver, String parametro,
                               String elemento, BloqueBasico bloque,
                               double inicial, double ultimo,
                               double incremento) {
    Resultados ventanaResultados = new Resultados(bloqueaResolver, parametro,
                                                  opcionesResolucion, elemento,
                                                  bloque, inicial, ultimo,
                                                  incremento, ventanaPrincipal);
    ventanaResultados.setSize(485, 335);
    Dimension tamanioAplicacion = ventanaPrincipal.getSize();
    Dimension tamanioOpcion = ventanaResultados.getSize();
    Point loc = ventanaPrincipal.getLocation();
    ventanaResultados.setLocation( (tamanioAplicacion.width -
                                    tamanioOpcion.width) / 2 + loc.x,
                                  (tamanioAplicacion.height -
                                   tamanioOpcion.height) / 2 + loc.y);
    ventanaResultados.setVisible(true);
    ventanaResultados.pack();

    ventanaResultados.setSize(485, 335);
    ventanaResultados.setVisible(true);
  }

  private void rellenarListaElementos() {
    // se introducen los elementos evaluables en el combo Elementos
    Bloque bloqueaEvaluar = bloquePadre;

    comboElementos.addItem("Tiempo");
    elemento = "Tiempo";

    // Luego el resto de bloques expansibles
    Iterator iterador = bloqueaEvaluar.componentes.iterator();
    anadirHijos(iterador);
    actualizarComboParametros();
  }

  /**Funcion que recibe el nombre de un bloque y busca entre el otro parametro
   * bloque y sus componentes el bloque. Ha de existir el bloque cuyo nombre
   * buscamos, en caso contrario devolver null. Solo busca entre los bloques
   * simples*/
  private BloqueBasico encontrarBloque(String nombre, Bloque bloque) {
    Iterator iterador = bloque.componentes.iterator();
    BloqueBasico bloqueDevuelto = null;
    while (iterador.hasNext()) {
      Bloque bloqueActual = (Bloque) iterador.next();
      // Si es extensible controlamos sus hijos
      if (bloqueActual.esExtensible()) {
        bloqueDevuelto = encontrarBloque(nombre, bloqueActual);
        // si ya lo hemos encontrado
        if (bloqueDevuelto != null) {
          break;
        }
      }
      else if (bloqueActual.nombre() == nombre) {
        bloqueDevuelto = (BloqueBasico) bloqueActual;
        break;
      }
    }
    return bloqueDevuelto;
  }

  private void actualizarComboParametros() {
    if (comboElementos.getSelectedItem() == "Tiempo") {
      comboParametros.removeAllItems();
      comboParametros.setEnabled(false);
    }
    else {
      comboParametros.removeAllItems();
      comboParametros.setEnabled(true);
      BloqueBasico bloqueaVariar =
          (BloqueBasico) encontrarBloque(elemento, bloquePadre);
      comboParametros.addItem("Tasa de fallos");
      comboParametros.addItem("MTTF");
      if (opcionesResolucion.esReparable()) {
        comboParametros.addItem("Tasa de reparacin");
        comboParametros.addItem("MTTR");
      }
    }
  }

  private void anadirHijos(Iterator iterador) {
    while (iterador.hasNext()) {
      Bloque bloqueActual = (Bloque) iterador.next();
      // Si es extensible aado los no extensibles de entre los hijos
      if (bloqueActual.esExtensible()) {
        Iterator iteradorHijo = bloqueActual.componentes.iterator();
        anadirHijos(iteradorHijo);
      }
      // si no es extensible lo aado a el
      else {
        try {
          // Intentar borrar el elemento para evitar que si ya existe
          // se introduzca dos veces
          comboElementos.removeItem(bloqueActual.nombre());
        } catch (Exception e) {
          // Si hay excepcin, es que el elemento no exista
        }
        comboElementos.addItem(bloqueActual.nombre());
      }
    }
  }

  private boolean comprobarValores() {
    boolean valido = true;

    if ( ( (Number) campoValorInicial.getValue()) == null ||
        ( (Number) campoValorFinal.getValue()) == null ||
        ( (Number) campoIncremento.getValue()) == null) {
      JOptionPane.showMessageDialog(null,
          "Ha de introducir los valores inicial, final e incremento para poder" +
          " realizar los clculos",
          "Faltan parmetros", JOptionPane.ERROR_MESSAGE);
      valido = false;
    }

    if (valido == true &&
        ( ( (Number) campoValorInicial.getValue()).doubleValue() < 0 ||
         ( (Number) campoValorFinal.getValue()).doubleValue() < 0 ||
         ( (Number) campoIncremento.getValue()).doubleValue() < 0)) {
      JOptionPane.showMessageDialog(null,
          "Los valores inicial, final e incremento han de ser positivos",
          "Valores errneos", JOptionPane.ERROR_MESSAGE);
      valido = false;
    }

    if (valido == true &&
        ( (Number) campoValorInicial.getValue()).doubleValue() >
        ( (Number) campoValorFinal.getValue()).doubleValue()) {
      JOptionPane.showMessageDialog(null,
          "El valor inicial no puede ser mayor que el final",
          "Valores errneos", JOptionPane.ERROR_MESSAGE);
      valido = false;
    }

    if (valido == true &&
        ( (Number) campoIncremento.getValue()).doubleValue() <= 0) {
      JOptionPane.showMessageDialog(null,
          "El valor del incremento ha de ser mayor que cero",
          "Valores errneos", JOptionPane.ERROR_MESSAGE);
      valido = false;

    }
    return valido;
  }

  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == botonAceptar) {
      if (comprobarValores()) {
        dispose();
        resolver(bloquePadre, elemento, encontrarBloque(elemento, bloquePadre),
                 (String) comboParametros.getSelectedItem());
      }
    }
    else if (e.getSource() == botonCancelar)
      dispose();
  }
}
