package interfaz.deshacer;

import javax.swing.undo.*;
import interfaz.Diagrama;
import bloques.Bloque;
import interfaz.Opciones;
import javax.swing.DefaultListModel;

/**
 * <p>Title: EditarCortarEdit</p>
 * <p>Description: Clase para la funcionalidad deshacer-rehacer de Cortar</p>
 * @author Diego Daz Garca
 * @version 1.0
 */

public class EditarCortarEdit
    extends AbstractUndoableEdit {
  Bloque bloqueCortado;
  Bloque padre;
  int posicion;
  Diagrama diagrama;
  Opciones opcionesResolucion;
  String[] listaEvaluables;
  String[] listaElegidos;

  public EditarCortarEdit (Bloque padre, Bloque bloqueCortado,
                           Diagrama diagrama, Opciones opcionesResolucion) {
    this.bloqueCortado = bloqueCortado;
    this.padre = padre;
    this.diagrama = diagrama;
    this.posicion = this.padre.componentes.indexOf(this.bloqueCortado);
    this.opcionesResolucion = opcionesResolucion;

    int numEvaluables = opcionesResolucion.obtenerListaEvaluables().size();
    listaEvaluables = new String[numEvaluables];
    opcionesResolucion.obtenerListaEvaluables().copyInto(listaEvaluables);

    int numElegidos = opcionesResolucion.obtenerListaElegidos().size();
    listaElegidos = new String[numElegidos];
    opcionesResolucion.obtenerListaElegidos().copyInto(listaElegidos);

    opcionesResolucion.eliminar(bloqueCortado);
  }

  public void undo() throws CannotUndoException {
    padre.add(bloqueCortado, posicion);
    diagrama.asignarPuedePegar(false);
    diagrama.cambiarBloqueMostrado(padre);
    opcionesResolucion.asignarListaEvaluables(listaEvaluables);
    opcionesResolucion.asignarListaElegidos(listaElegidos);
  }

  public void redo() throws CannotUndoException {
    diagrama.copiarBloques(bloqueCortado, true);
    diagrama.cambiarBloqueMostrado(padre);
  }

  public boolean canUndo() {
    return true;
  }

  public boolean canRedo() {
    return true;
  }

  public String getPresentationName() {
    return "Cortar";
  }
}
