package interfaz.deshacer;

import bloques.*;
import javax.swing.undo.*;
import java.util.*;
import interfaz.FiguraBloque;
import interfaz.Diagrama;
import interfaz.Opciones;
import javax.swing.DefaultListModel;

/**
 * <p>Title: ElementoEliminarEdit</p>
 * <p>Description: Clase para la funcionalidad deshacer-rehacer de Eliminar
 * bloque</p>
 * @author Diego Daz Garca
 * @version 1.0
 */

public class ElementoEliminarEdit
    extends AbstractUndoableEdit {
  int posicion;
  Bloque bloqueEliminado;
  Bloque bloquePadre;
  Diagrama diagramaVisible;
  Opciones opcionesResolucion;
  String[] listaEvaluables;
  String[] listaElegidos;

  public ElementoEliminarEdit(Bloque modelo, int posicion_,
                              Diagrama diagramaVisible,
                              Opciones opcionesResolucion_) {
    this.diagramaVisible = diagramaVisible;
    posicion = posicion_;
    bloquePadre = modelo;
    bloqueEliminado = (Bloque) modelo.componentes.get(posicion);
    opcionesResolucion = opcionesResolucion_;

    int numEvaluables = opcionesResolucion.obtenerListaEvaluables().size();
    listaEvaluables = new String[numEvaluables];
    opcionesResolucion.obtenerListaEvaluables().copyInto(listaEvaluables);

    int numElegidos = opcionesResolucion.obtenerListaElegidos().size();
    listaElegidos = new String[numElegidos];
    opcionesResolucion.obtenerListaElegidos().copyInto(listaElegidos);

    opcionesResolucion.eliminar(bloqueEliminado);
  }

  public void undo() throws CannotUndoException {
    bloquePadre.add(bloqueEliminado, posicion);
    diagramaVisible.cambiarBloqueMostrado(bloquePadre);
    opcionesResolucion.asignarListaEvaluables(listaEvaluables);
    opcionesResolucion.asignarListaElegidos(listaElegidos);
  }

  public void redo() throws CannotUndoException {
    bloquePadre.eliminaBloque(posicion);
    diagramaVisible.cambiarBloqueMostrado(bloquePadre);
    opcionesResolucion.eliminar(bloqueEliminado);
  }

  public boolean canUndo() {
    return true;
  }

  public boolean canRedo() {
    return true;
  }

  public String getPresentationName() {
    return "Eliminar bloque";
  }
}
