package interfaz;

import java.util.Observer;
//import javax.swing.tree.DefaultMutableTreeNode;
import java.util.Observable;
import bloques.Bloque;
import util.bloqueIndice;
import java.util.Iterator;
import javax.swing.*;
import interfaz.deshacer.*;
import javax.swing.undo.*;
import javax.swing.tree.*;

/**
 * <p>Title: Nodo.java</p>
 * <p>Description: Clase para los nodos que forman el rbol de bloques</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Diego Daz Garca
 * @version 1.0
 */

class nodo
    extends DefaultMutableTreeNode
    implements Observer {
  Bloque modelo;
  arbolBloques arbol;

  /**Constructor
   * @param abrirNuevo indica si el nodo a crear procede de la opcin abrir
   * o nuevo diagrama, falso si es el resultado de aadir, pegar bloque
   * @param bloque Bloque al que va a representar el nodo
   * @param arbol Arbol en el que est representado el nodo*/
  public nodo(Bloque bloque, boolean abrirNuevo, arbolBloques arbol) {
    super(bloque);
    modelo = bloque;
    bloque.addObserver(this);
    this.arbol = arbol;

    // Caso de que el nodo tenga hijos y no tenga padre es especial (abrir)
    if (abrirNuevo == true) {
      sumarHijosAbrir();
    }
  }

  /**Aade los hijos del bloquePrincipal al nodo al abrir un bloque*/
  public void sumarHijosAbrir() {
    Iterator iterador = modelo.componentes.iterator();
    while (iterador.hasNext()) {
      Bloque bloqueHijo = (Bloque) iterador.next();
      nodo nodoHijo = new nodo(bloqueHijo, true, arbol);
      this.add(nodoHijo);
    }
  }

  /**Funcin para el modelo MVC que actualiza la vista
   * @param t Objeto observable que produjo la llamada
   * @param o Objeto que se puede pasar con el update*/
  public void update(Observable t, Object o) {
    // Si hay un evento que cambie la estructura del arbol, el objeto Observable
    // enviar un bloqueIndice
    bloqueIndice bI = null;
    try {
      bI = (bloqueIndice) o;
    } catch (Exception e) {
    }
    if (bI != null) { // Slo aadir bloque y quitar bloque envian el objeto Bloque nuevo
      Bloque nuevoBloque;
      // Ser null si se ha eliminado un bloque
      if (bI.obtenerBloque() == null) {
        int num = ((DefaultMutableTreeNode)this).getChildCount();
        DefaultMutableTreeNode nodo = (DefaultMutableTreeNode) this.getChildAt(bI.obtenerIndice());
        arbol.modeloArbol.removeNodeFromParent(nodo);
      }
      else {
        anadirNuevosHijos(bI);
      }
    }
    arbol.repaint();
  }

  /**Comprueba si el nodo es hoja
   * @return Cierto si es hoja falso si no*/
  public boolean isLeaf() {
    return !modelo.esExtensible();
  }

  /**Se lanza al aadir nuevos hijos a un bloque para que se actualice el
   * rbol
   * @param bI bloqueIndice con el bloque creado y el lugar que ocupa en el
   * vector de hijos del padre*/
  public void anadirNuevosHijos(bloqueIndice bI) {
    Bloque nuevoBloque = bI.obtenerBloque();
    nodo hijo = new nodo(nuevoBloque, false, arbol);
    arbol.modeloArbol.insertNodeInto(hijo, this, bI.obtenerIndice());

    if (nuevoBloque.esExtensible()) {
      for (int i = 0; i < nuevoBloque.componentes.size(); i++) {
        bloqueIndice nuevoBI = new bloqueIndice(
            (Bloque) nuevoBloque.componentes.get(i), i);
        hijo.anadirNuevosHijos(nuevoBI);
      }
    }
  }

  /**Asignamos al nodo el ArbolBloques en el que est dibujado
   * @param arbol El arbolBloques en el que est representado el rbol*/
  public void asignarModeloArbol(arbolBloques arbol) {
    this.arbol = arbol;
  }

  /**Actualiza el modelo en caso de un cambio de nombre en el nodo que lo
   * representa*/
  public void actualizaNombre() {
    // En el uso del userObject hay que tener en cuenta que si se cambia el
    // nombre del nodo modificndolo directamente en el rbol (por ejemplo, con
    // F2), el DefaultMutableTreeNode crea un objeto nuevo de tipo String. Por
    // eso hay que restaurar el userObject
    String nombreNuevo = this.getUserObject().toString();
    this.userObject = modelo;

    if (!modelo.nombreValido(nombreNuevo)) {
      JOptionPane.showMessageDialog(null,
                                    "Elija otro nombre, ya existe otro diagrama con el nombre "
                                    + nombreNuevo,
                                    "Nombre duplicado",
                                    JOptionPane.ERROR_MESSAGE);
      return;
    }

    String nombreViejo = modelo.nombre();
    modelo.cambiarNombre(nombreNuevo);

    FramePrincipal ventanaPrincipal = arbol.obtenerVentanaPrincipal();
    UndoableEdit edit = new ElementoCambiarNombreEdit(ventanaPrincipal,
        modelo, nombreViejo, nombreNuevo);
    ventanaPrincipal.gestionarCambioNombre(nombreViejo, nombreNuevo);

    // notifico al escuchador de eventos de deshacer rehacer
    interfaz.FramePrincipal.undoSupport_.postEdit(edit);
  }
}
